#include <sstream>
#include <lemon/lp_skeleton.h>
#include "test_tools.h"


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_GLPK
#include <lemon/lp_glpk.h>
#endif

#ifdef HAVE_CPLEX
#include <lemon/lp_cplex.h>
#endif

using namespace lemon;

void lpTest(LpSolverBase & lp)
{



  typedef LpSolverBase LP;

  std::vector<LP::Col> x(10);
  //  for(int i=0;i<10;i++) x.push_back(lp.addCol());
  lp.addColSet(x);

#ifndef GYORSITAS

  std::vector<LP::Col> y(10);
  lp.addColSet(y);

  std::map<int,LP::Col> z;
  
  z.insert(std::make_pair(12,INVALID));
  z.insert(std::make_pair(2,INVALID));
  z.insert(std::make_pair(7,INVALID));
  z.insert(std::make_pair(5,INVALID));
  
  lp.addColSet(z);

  {
    LP::Expr e,f,g;
    LP::Col p1,p2,p3,p4,p5;
    LP::Constr c;
    
    p1=lp.addCol();
    p2=lp.addCol();
    p3=lp.addCol();
    p4=lp.addCol();
    p5=lp.addCol();
    
    e[p1]=2;
    e.constComp()=12;
    e[p1]+=2;
    e.constComp()+=12;
    e[p1]-=2;
    e.constComp()-=12;
    
    e=2;
    e=2.2;
    e=p1;
    e=f;
    
    e+=2;
    e+=2.2;
    e+=p1;
    e+=f;
    
    e-=2;
    e-=2.2;
    e-=p1;
    e-=f;
    
    e*=2;
    e*=2.2;
    e/=2;
    e/=2.2;
    
    e=((p1+p2)+(p1-p2)+(p1+12)+(12+p1)+(p1-12)+(12-p1)+
       (f+12)+(12+f)+(p1+f)+(f+p1)+(f+g)+
       (f-12)+(12-f)+(p1-f)+(f-p1)+(f-g)+
       2.2*f+f*2.2+f/2.2+
       2*f+f*2+f/2+
       2.2*p1+p1*2.2+p1/2.2+
       2*p1+p1*2+p1/2
       );


    c = (e  <= f  );
    c = (e  <= 2.2);
    c = (e  <= 2  );
    c = (e  <= p1 );
    c = (2.2<= f  );
    c = (2  <= f  );
    c = (p1 <= f  );
    c = (p1 <= p2 );
    c = (p1 <= 2.2);
    c = (p1 <= 2  );
    c = (2.2<= p2 );
    c = (2  <= p2 );
    
    c = (e  >= f  );
    c = (e  >= 2.2);
    c = (e  >= 2  );
    c = (e  >= p1 );
    c = (2.2>= f  );
    c = (2  >= f  );
    c = (p1 >= f  );
    c = (p1 >= p2 );
    c = (p1 >= 2.2);
    c = (p1 >= 2  );
    c = (2.2>= p2 );
    c = (2  >= p2 );
    
    c = (e  == f  );
    c = (e  == 2.2);
    c = (e  == 2  );
    c = (e  == p1 );
    c = (2.2== f  );
    c = (2  == f  );
    c = (p1 == f  );
    //c = (p1 == p2 );
    c = (p1 == 2.2);
    c = (p1 == 2  );
    c = (2.2== p2 );
    c = (2  == p2 );
    
    c = (2 <= e <= 3);
    c = (2 <= p1<= 3);
    
    c = (2 >= e >= 3);
    c = (2 >= p1>= 3);
    
    e[x[3]]=2;
    e[x[3]]=4;
    e[x[3]]=1;
    e.constComp()=12;
    
    lp.addRow(LP::INF,e,23);
    lp.addRow(LP::INF,3.0*(x[1]+x[2]/2)-x[3],23);
    lp.addRow(LP::INF,3.0*(x[1]+x[2]*2-5*x[3]+12-x[4]/3)+2*x[4]-4,23);
    
    lp.addRow(x[1]+x[3]<=x[5]-3);
    lp.addRow(-7<=x[1]+x[3]-12<=3);
    lp.addRow(x[1]<=x[5]);
  }
  
  {
    LP::DualExpr e,f,g;
    LP::Row p1,p2,p3,p4,p5;
    
    e[p1]=2;
    e[p1]+=2;
    e[p1]-=2;
    
    e=p1;
    e=f;
    
    e+=p1;
    e+=f;
    
    e-=p1;
    e-=f;
    
    e*=2;
    e*=2.2;
    e/=2;
    e/=2.2;
    
    e=((p1+p2)+(p1-p2)+
       (p1+f)+(f+p1)+(f+g)+
       (p1-f)+(f-p1)+(f-g)+
       2.2*f+f*2.2+f/2.2+
       2*f+f*2+f/2+
       2.2*p1+p1*2.2+p1/2.2+
       2*p1+p1*2+p1/2
       );
  }
  
#endif
}

void solveAndCheck(LpSolverBase& lp, LpSolverBase::SolutionStatus stat, 
		   double exp_opt) {
  using std::string;
  lp.solve();
  //int decimal,sign;
  std::ostringstream buf;
  buf << "Primalstatus should be: " << int(stat);

  //  itoa(stat,buf1, 10);
  check(lp.primalStatus()==stat, buf.str());

  if (stat ==  LpSolverBase::OPTIMAL) {
    std::ostringstream buf;
    buf << "Wrong optimal value: the right optimum is " << exp_opt; 
    check(std::abs(lp.primalValue()-exp_opt) < 1e-3, buf.str());
    //+ecvt(exp_opt,2)
  }
}
 
void aTest(LpSolverBase & lp)
{
  typedef LpSolverBase LP;

 //The following example is very simple

  typedef LpSolverBase::Row Row;
  typedef LpSolverBase::Col Col;


  Col x1 = lp.addCol();
  Col x2 = lp.addCol();


  //Constraints
  Row upright=lp.addRow(x1+x2 <=1);  
  lp.addRow(x1+x2 >=-1);  
  lp.addRow(x1-x2 <=1);  
  lp.addRow(x1-x2 >=-1);  
  //Nonnegativity of the variables
  lp.colLowerBound(x1, 0);
  lp.colLowerBound(x2, 0);
  //Objective function
  lp.setObj(x1+x2);

  lp.max();

  //Maximization of x1+x2
  //over the triangle with vertices (0,0) (0,1) (1,0)
  double expected_opt=1;
  solveAndCheck(lp, LpSolverBase::OPTIMAL, expected_opt);
  
  //Minimization
  lp.min();
  expected_opt=0;
  solveAndCheck(lp, LpSolverBase::OPTIMAL, expected_opt);
  
  //Vertex (-1,0) instead of (0,0)
  lp.colLowerBound(x1, -LpSolverBase::INF);
  expected_opt=-1;
  solveAndCheck(lp, LpSolverBase::OPTIMAL, expected_opt);

  //Erase one constraint and return to maximization
  lp.eraseRow(upright);
  lp.max();
  expected_opt=LpSolverBase::INF;
  solveAndCheck(lp, LpSolverBase::INFINITE, expected_opt);

  //Infeasibilty
  lp.addRow(x1+x2 <=-2);  
  solveAndCheck(lp, LpSolverBase::INFEASIBLE, expected_opt);

  //Change problem and forget to solve
  lp.min();
  check(lp.primalStatus()==LpSolverBase::UNDEFINED,"Primalstatus should be UNDEFINED");

//   lp.solve();
//   if (lp.primalStatus()==LpSolverBase::OPTIMAL){
//     std::cout<< "Z = "<<lp.primalValue()
// 	     << " (error = " << lp.primalValue()-expected_opt
// 	     << "); x1 = "<<lp.primal(x1)
// 	     << "; x2 = "<<lp.primal(x2)
// 	     <<std::endl;
    
//   }
//   else{
//     std::cout<<lp.primalStatus()<<std::endl;
//     std::cout<<"Optimal solution not found!"<<std::endl;
//   }

 

}


int main() 
{
  LpSkeleton lp_skel;
  lpTest(lp_skel);

#ifdef HAVE_GLPK
  LpGlpk lp_glpk1,lp_glpk2;
  lpTest(lp_glpk1);
  aTest(lp_glpk2);
#endif

#ifdef HAVE_CPLEX
  LpCplex lp_cplex;
  lpTest(lp_cplex);
  aTest(lp_cplex);
#endif

  return 0;
}
